with Gtk.Widget; use Gtk.Widget;
with Glib; use Glib;
with Auction_Interface; use Auction_Interface;
--  with Buy_Command_Types; use Buy_Command_Types;
with Debug_Auction; use Debug_Auction;
with Gdk.Bitmap; use Gdk.Bitmap;
with Gtk.Main; use Gtk.Main;
with Timeout_Bid_Win;
with Notenoughfunds_Dialog_Pkg; use Notenoughfunds_Dialog_Pkg;
with Ada.Exceptions; use Ada.Exceptions;

package body Bid_Window_Pkg.Callbacks is

   use Gtk.Arguments;

   ---------------------------
   -- On_Bid_Window_Destroy --
   ---------------------------

   procedure On_Bid_Window_Destroy
     (Object : access Gtk_Window_Record'Class)
   is
   begin
      Timeout_Remove
        (Bid_Window_Access (Object) .My_Timeout_Handler_Id);
   end On_Bid_Window_Destroy;

   --------------------------------
   -- On_Bid_Window_Delete_Event --
   --------------------------------

   function On_Bid_Window_Delete_Event
     (Object : access Gtk_Widget_Record'Class;
      Params : Gtk.Arguments.Gtk_Args) return Boolean
   is
   begin
      return False;
   end On_Bid_Window_Delete_Event;

   ---------------------------
   -- On_Bid_Button_Pressed --
   ---------------------------

   procedure On_Bid_Button_Pressed
     (Object : access Gtk_Button_Record'Class;
      Bid_Win : Bid_Window_Access) is
   begin
      if Bid_Win.Auction_State /= Bidding then
         Timeout_Remove (Bid_Win.My_Timeout_Handler_Id);
         Destroy_Cb (Bid_Win);
      else
         declare
            Value : Integer :=
              Integer (Get_Value_As_Int (Bid_Win.Bid_Spinbutton));
         begin
            Place_Bid (Bid_Win.Auction_Id, Bid_Win.User_Id, Value);
         exception
            when E : others =>
               declare
                  Notenoughfunds_Dialog : Notenoughfunds_Dialog_Access;
               begin
                  Debug_Auction.Put_Line ("Exception raised: "
                                          & Exception_Name (E));
                  Gtk_New (Notenoughfunds_Dialog);
                  Show_All (Notenoughfunds_Dialog);
               end;
         end;
      end if;
      Put_Line ("Leaving Bid pressed");
   end On_Bid_Button_Pressed;

   function Update_Infos (Bid_Win : Bid_Window_Access) return Boolean is

      Current_Bid : Natural;
      Current_Status : Status_Type;
      Value : Integer := Integer (Get_Value_As_Int (Bid_Win.Bid_Spinbutton));
      Adj : Gtk_Adjustment;


   begin
      Put_Line ("Calling status");
      Get_Status (Bid_Win.Auction_Id, Bid_Win.User_Id,
                  Current_Bid, Current_Status);
      Put_Line ("Status: " & Status_Type'Image (Current_Status));
      case Current_Status is
         when Active =>
            Set (Bid_Win.The_Pixmap,
                 Get_Pixmap (Bid_Win.Sickface_Pixmap),
                 Get_Mask (Bid_Win.Sickface_Pixmap));
            if Bid_Win.Has_Placed_Bid then
               Set_Text (Bid_Win.Label33,
                         "Someone has placed a higher bid!"
                         & ASCII.LF & "Bid again!");
            else
               Set_Text (Bid_Win.Label33,
                         "Enter your bid and click the bid button!"
                         & ASCII.LF);
            end if;
            Set_Text (Gtk_Label (Get_Child (Bid_Win.Bid_Button)), "Bid");
            Show (Bid_Win.Bid_Spinbutton);
            Bid_Win.Button_Is_Quit := False;
            Bid_Win.Auction_State := Bidding;
         when Current_Winner =>
            --  Bid_Win.The_Pixmap := Bid_Win.Smile_Pixmap;
            --  Set_Label (Bid_Win.Bid_Button, "Quit");
            -- Initialize (Bid_Win.Bid_Button, "Quit");
            Set (Bid_Win.The_Pixmap,
                 Get_Pixmap (Bid_Win.Smiley_Pixmap),
                 Get_Mask (Bid_Win.Smiley_Pixmap));
            --   Draw (Bid_Win.The_Pixmap);
            Set_Text (Bid_Win.Label33,
                      "You have currently placed " & ASCII.LF
                      & "the highest bid!");
            Set_Text (Gtk_Label (Get_Child (Bid_Win.Bid_Button)), "Quit");
            Hide (Bid_Win.Bid_Spinbutton);
            Bid_Win.Button_Is_Quit := True;
            Bid_Win.Has_Placed_Bid := True;
            Bid_Win.Auction_State := Currently_Highest;
         when Winner =>
            Set (Bid_Win.The_Pixmap,
                 Get_Pixmap (Bid_Win.Smiley3_Pixmap),
                 Get_Mask (Bid_Win.Smiley3_Pixmap));
            Set_Text (Gtk_Label (Get_Child (Bid_Win.Bid_Button)), "Quit");
            Hide (Bid_Win.Bid_Spinbutton);
            Bid_Win.Button_Is_Quit := True;
            Set_Text (Bid_Win.Label33,
                      "Auction completed." & ASCII.LF & "You are the winner!");
            Bid_Win.Auction_State := Finished;
            Timeout_Remove (Bid_Win.My_Timeout_Handler_Id);
         when Loser =>
            Set (Bid_Win.The_Pixmap,
                 Get_Pixmap (Bid_Win.Stop_Pixmap),
                 Get_Mask (Bid_Win.Stop_Pixmap));
            Set_Text (Gtk_Label (Get_Child (Bid_Win.Bid_Button)), "Quit");
            Hide (Bid_Win.Bid_Spinbutton);
            Bid_Win.Button_Is_Quit := True;
            Set_Text (Bid_Win.Label33,
                      "Auction completed." & ASCII.LF & "Sorry, you loose..");
            Bid_Win.Auction_State := Finished;
            Timeout_Remove (Bid_Win.My_Timeout_Handler_Id);
      end case;
      Queue_Draw (Bid_Win.The_Pixmap);

      Set_Text (Bid_Win.Current_Bid_Label, Natural'Image (Current_Bid));
      if Value <= Current_Bid then
         Set_Value (Bid_Win.Bid_Spinbutton, Gfloat (Current_Bid + 1));
         Adj := Get_Adjustment (Bid_Win.Bid_Spinbutton);
         Set_Lower (Adj, Gfloat (Current_Bid + 1));
         Set_Upper (Adj, Gfloat (Current_Bid + 5001));
         Set_Adjustment (Bid_Win.Bid_Spinbutton, Adj);
      end if;
      Put_Line ("Finished update info");
      return True;
   end Update_Infos;


end Bid_Window_Pkg.Callbacks;
