with Debug_Auction;
package body Auction_Communication is

   protected body Auction_Communication_Type is

      entry Called (Operation : out Auction_Operation_Type)
        when Current_Operation /= Nothing is
      begin
         Operation := Current_Operation;
         if Current_Operation = Accept_Bid then
            Current_Operation := Nothing;
         end if;
      end Called;

      entry Get_Current_Bid_Start when Current_Operation = Nothing is
      begin
         Current_Operation := Get_Current_Bid;
         Current_Bid := -1;
      end Get_Current_Bid_Start;

      procedure Set_Current_Bid (Bid : Natural) is
      begin
         Current_Bid := Bid;
         Current_Operation := Nothing;
      end Set_Current_Bid;

      entry Get_Current_Bid_Result (Bid : out Natural)
        when Current_Bid >= 0 is
      begin
         Bid := Current_Bid;
      end Get_Current_Bid_Result;

      entry Get_Status_Start when Current_Operation = Nothing is
      begin
         Current_Operation := Get_Status;
         Current_Bid := -1;
         Current_Status := Active;
      end Get_Status_Start;

      procedure Set_Status (Bid : Natural;
                            State : Auction_Status) is
      begin
         Current_Bid := Bid;
         Current_Status := State;
         Current_Operation := Nothing;
      end Set_Status;

      entry Get_Status_Result (Bid : out Natural;
                               State : out Auction_Status)
        when Current_Bid >= 0 is
      begin
         Bid := Current_Bid;
         State := Current_Status;
      end Get_Status_Result;


      entry Get_Initial_Bid_Start
        when Current_Operation = Nothing is
      begin
         if Initial_Bid < 0 then
            Current_Operation := Get_Initial_Bid;
         end if;
      end Get_Initial_Bid_Start;

      procedure Set_Initial_Bid (Bid : Natural) is
      begin
         Initial_Bid := Bid;
         Current_Operation := Nothing;
      end Set_Initial_Bid;

      entry Get_Initial_Bid_Result (Bid : out Natural)
        when Initial_Bid >= 0 is
      begin
         Bid := Initial_Bid;
         Current_Operation := Nothing;
      end Get_Initial_Bid_Result;

      entry Get_Auction_Name_Start
         when Current_Operation = Nothing is
      begin
         if Current_Name = null then
            Current_Operation := Get_Auction_Name;
         end if;
      end Get_Auction_Name_Start;

      procedure Set_Auction_Name (Name : String_Ref) is
      begin
         Current_Name := Name;
         Current_Operation := Nothing;
      end Set_Auction_Name;

      entry Get_Auction_Name_Result (Name : out String_Ref)
        when Current_Name /= null is
      begin
         Name := Current_Name;
      end Get_Auction_Name_Result;

      entry Get_Auction_Description_Start
         when Current_Operation = Nothing is
      begin
         if Current_Description = null then
            Current_Operation := Get_Auction_Description;
         end if;
      end Get_Auction_Description_Start;

      procedure Set_Auction_Description (Description : String_Ref) is
      begin
         Current_Description := Description;
         Current_Operation := Nothing;
      end Set_Auction_Description;

      entry Get_Auction_Description_Result (Description : out String_Ref)
        when Current_Description /= null is
      begin
         Description := Current_Description;
      end Get_Auction_Description_Result;

      entry Place_Bid_Start (Bid : in Natural)
        when Current_Operation = Nothing is
      begin
         Bid_To_Place := Bid;
         Current_Operation := Place_Bid;
         Bid_Accepted := Pending;
      end Place_Bid_Start;

      entry Place_Bid_Result when
        Bid_Accepted /= Pending is
      begin
         if Bid_Accepted /= Yes then
            raise Not_Enough_Funds;
         end if;
      end Place_Bid_Result;

      procedure Get_Bid (Bid : out Natural) is
      begin
         Bid := Bid_To_Place;
      end Get_Bid;

      procedure Place_Bid_Complete (OK : Boolean) is
      begin
         Current_Operation := Nothing;
         if OK then
            Bid_Accepted := Yes;
         else
            Bid_Accepted := No;
         end if;
      end Place_Bid_Complete;

      entry Accept_Bid when Current_Operation = Nothing is
      begin
         Current_Operation := Accept_Bid;
      end Accept_Bid;

   end Auction_Communication_Type;

end Auction_Communication;
