with My_Strings; use My_Strings;
with Auctions; use Auctions;

package Auction_Communication is

   type Auction_Operation_Type is
      (Nothing, Get_Current_Bid, Get_Initial_Bid, Get_Status,
       Get_Auction_Name, Get_Auction_Description, Place_Bid, Accept_Bid, Bid);

   type Place_Bid_Status is (Yes, No, Pending);

   Not_Enough_Funds : exception;

   protected type Auction_Communication_Type is

      entry Called (Operation : out Auction_Operation_Type);

      entry Get_Current_Bid_Start;
      procedure Set_Current_Bid (Bid : Natural);
      entry Get_Current_Bid_Result (Bid : out Natural);

      entry Get_Status_Start;
      procedure Set_Status (Bid : Natural; State : Auction_Status);
      entry Get_Status_Result (Bid : out Natural; State : out Auction_Status);

      entry Get_Initial_Bid_Start;
      procedure Set_Initial_Bid (Bid : Natural);
      entry Get_Initial_Bid_Result (Bid : out Natural);

      entry Get_Auction_Name_Start;
      procedure Set_Auction_Name (Name : String_Ref);
      entry Get_Auction_Name_Result (Name : out String_Ref);

      entry Get_Auction_Description_Start;
      procedure Set_Auction_Description (Description : String_Ref);
      entry Get_Auction_Description_Result (Description : out String_Ref);

      entry Place_Bid_Start (Bid : in Natural);
      procedure Get_Bid (Bid : out Natural);
      procedure Place_Bid_Complete (OK : Boolean);
      entry Place_Bid_Result;

      entry Accept_Bid;

   private

      Current_Operation : Auction_Operation_Type := Nothing;

      Current_Bid : Integer := -1;
      Initial_Bid : Integer := -1;

      Current_Name : String_Ref;
      Current_Description : String_Ref;
      Current_Status : Auction_Status;

      Bid_To_Place : Natural;
      Bid_Accepted : Place_Bid_Status;

   end Auction_Communication_Type;

   type Auction_Communication_Ref is access all Auction_Communication_Type;

end Auction_Communication;
