with Auction_Objects; use Auction_Objects;
with Auction_Communication; use Auction_Communication;
with My_Strings; use My_Strings;
with Debug_Auction;
with Auctions; use Auctions;

package body Auction_Interface is

   function Get_Current_Bid (Auction : in Auction_Identifier_Type;
                             User : in User_Identifier_Type)
      return Natural is
      Result : Natural;
      Bidder : Bidder_Node_Ref := Auction_Table (Auction).Bidder_Nodes;
   begin
      if Auction_Table (Auction).User_Id = User then
         Auction_Table (Auction).Communication.Get_Current_Bid_Start;
         Auction_Table (Auction).Communication.Get_Current_Bid_Result (Result);
         return Result;
      else
         while Bidder /= null loop
            if Bidder.User = User then
               Bidder.Bidder_Communication.Get_Current_Bid_Start;
               Bidder.Bidder_Communication.Get_Current_Bid_Result (Result);
               return Result;
            end if;
            Bidder := Bidder.Next;
         end loop;
         Auction_Table (Auction).Communication.Get_Current_Bid_Start;
         Auction_Table (Auction).Communication.Get_Current_Bid_Result (Result);
         return Result;
      end if;
   end Get_Current_Bid;

   procedure Get_Status (Auction : in Auction_Identifier_Type;
                         User : in User_Identifier_Type;
                         Current_Bid : out Natural;
                         Status : out Status_Type) is
      Bidder : Bidder_Node_Ref := Auction_Table (Auction).Bidder_Nodes;
      Temp_Status : Auction_Status;
   begin
      while Bidder /= null loop
         if Bidder.User = User then
            Bidder.Bidder_Communication.Get_Status_Start;
            Bidder.Bidder_Communication.Get_Status_Result (Current_Bid, Temp_Status);
            case Temp_Status is
               when Active =>
                  Status := Active;
               when Current_Winner =>
                  Status := Current_Winner;
               when Winner =>
                  Status := Winner;
               when Loser =>
                  Status := Loser;
               when Accepted =>
                  Status := Loser;
            end case;
            return;
         end if;
         Bidder := Bidder.Next;
      end loop;
      raise Not_Joined;
   end Get_Status;

   function Get_Initial_Bid (Auction : in Auction_Identifier_Type;
                             User : in User_Identifier_Type)
                            return Natural is
      Result : Natural;
      Bidder : Bidder_Node_Ref := Auction_Table (Auction).Bidder_Nodes;
   begin
      if User = Auction_Table (Auction).User_Id then
         Auction_Table (Auction).Communication.Get_Initial_Bid_Start;
         Auction_Table (Auction).Communication.Get_Initial_Bid_Result (Result);
         return Result;
      else
         while Bidder /= null loop
            if Bidder.User = User then
               Bidder.Bidder_Communication.Get_Initial_Bid_Start;
               Bidder.Bidder_Communication.Get_Initial_Bid_Result (Result);
               return Result;
            end if;
            Bidder := Bidder.Next;
         end loop;
         Auction_Table (Auction).Communication.Get_Initial_Bid_Start;
         Auction_Table (Auction).Communication.Get_Initial_Bid_Result (Result);
         return Result;
      end if;
   end Get_Initial_Bid;

   function Get_Name (Auction : in Auction_Identifier_Type;
                      User : in User_Identifier_Type)
                     return String is
      Result : String_Ref;
      Bidder : Bidder_Node_Ref := Auction_Table (Auction).Bidder_Nodes;
   begin
      if Auction_Table (Auction).User_Id = User then
         Auction_Table (Auction).Communication.Get_Auction_Name_Start;
         Auction_Table (Auction).Communication.Get_Auction_Name_Result (Result);
         return Result.all;
      else
         while Bidder /= null loop
            if Bidder.User = User then
               Bidder.Bidder_Communication.Get_Auction_Name_Start;
               Bidder.Bidder_Communication.Get_Auction_Name_Result (Result);
               return Result.all;
            end if;
            Bidder := Bidder.Next;
         end loop;
         Auction_Table (Auction).Communication.Get_Auction_Name_Start;
         Auction_Table (Auction).Communication.Get_Auction_Name_Result (Result);
         return Result.all;
      end if;
   end Get_Name;

   function Get_Description (Auction : in Auction_Identifier_Type;
                            User : User_Identifier_Type)
                            return String is
      Result : String_Ref;
      Bidder : Bidder_Node_Ref := Auction_Table (Auction).Bidder_Nodes;
   begin
      if Auction_Table (Auction).User_Id = User then
         Auction_Table (Auction).Communication.Get_Auction_Description_Start;
         Auction_Table (Auction).Communication.Get_Auction_Description_Result (Result);
         return Result.all;
      else
         while Bidder /= null loop
            if Bidder.User = User then
               Bidder.Bidder_Communication.Get_Auction_Description_Start;
               Bidder.Bidder_Communication.Get_Auction_Description_Result (Result);
               return Result.all;
            end if;
            Bidder := Bidder.Next;
         end loop;
         Auction_Table (Auction).Communication.Get_Auction_Description_Start;
         Auction_Table (Auction).Communication.Get_Auction_Description_Result (Result);
         return Result.all;
      end if;
   end Get_Description;

   procedure Place_Bid (Auction : in Auction_Identifier_Type;
                        User : in User_Identifier_Type;
                        Bid : Natural) is
      Bidder : Bidder_Node_Ref := Auction_Table (Auction).Bidder_Nodes;
   begin
      while Bidder /= null loop
         if Bidder.User = User then
            Bidder.Bidder_Communication.Place_Bid_Start (Bid);
            Bidder.Bidder_Communication.Place_Bid_Result;
            return;
         end if;
         Bidder := Bidder.Next;
      end loop;
      raise Not_Joined;
   end Place_Bid;

   procedure Accept_Bid (Auction : in Auction_Identifier_Type;
                         User : in User_Identifier_Type) is
   begin
      if Auction_Table (Auction).User_Id = User then
         Auction_Table (Auction).Communication.Accept_Bid;
      end if;
   end Accept_Bid;

end Auction_Interface;

