with User_Tasks; use User_Tasks;
with User_Interface; use User_Interface;
with User_Objects; use User_Objects;
with Server_Utilities; use Server_Utilities;
with Debug_Auction;

with Transactional_Users; use Transactional_Users;
with Transactional_Accounts; use Transactional_Accounts;
with Transactional_Auctions; use Transactional_Auctions;
with Auction_Lists; use Auction_Lists;
with Object_Based_Transaction_Interface; use Object_Based_Transaction_Interface;

package body Server_Interface is

   function Login (Username : String;
                   Password : String) return User_Identifier_Type is

      User : Transactional_User_Ref;
      Account : Transactional_Account_Ref;
   begin
      Debug_Auction.Put (Username & " tries to login...");
      if Login_Correct (Username, Password) then
         Debug_Auction.Put_Line ("accepted.");
         for I in User_Table'Range loop
            if User_Table (I).Taken and then
              User_Table (I).Username.S = Username then
               return I;
            end if;
         end loop;

         --  user has never connected yet -> allocate a new communication

         for I in User_Table'Range loop
            if not User_Table (I).Taken then
               User_Table (I).Username := To_String_Type (Username);
               User_Table (I).Taken := True;

               declare
                  T : User_Task_Ref;
               begin
                  User := Get_The_User (Username, Password);
                  Account := Get_The_Account (User);
                  T := new User_Task_Type (User_Table (I).Communication'Access,
                                           new String' (Username),
                                           User, Account);
               end;

               return I;
            end if;
         end loop;
         raise Program_Error;
      else
         Debug_Auction.Put_Line ("refused.");
         raise Login_Incorrect;
      end if;
   end Login;

   procedure New_User (Name : String; Password : String) is
   begin
      Create_New_User (Name, Password);
   end New_User;

   function Get_Current_Auction_List return Remote_String_Array_Type is
   begin
      return The_Auctions.Get_All_Auctions_Titles;
   end Get_Current_Auction_List;

end Server_Interface;

