with My_Strings; use My_Strings;
with Auction_Communication; use Auction_Communication;
with Transactional_Auctions; use Transactional_Auctions;
--  with Auction_Interface; use Auction_Interface;
with Remote_String_Arrays; use Remote_String_Arrays;

package User_Communication is

   type User_Operation_Type is
      (Nothing, Deposit, Balance, Get_User_Name, Join_Auction, Start_Auction);

   protected type User_Communication_Type is

      entry Called (Operation : out User_Operation_Type);

      entry Get_Balance_Start;
      procedure Set_Balance (Value : Natural);
      entry Get_Balance_Result (Value : out Natural);

      entry Get_User_Name_Start;
      procedure Set_User_Name (Name : String_Ref);
      entry Get_User_Name_Result (Name : out String_Ref);

      entry Start_Auction (Communication : Auction_Communication_Ref;
                           Title, Description : in String;
                           Initial_Bid : in Integer;
                           Auction_Id : Auction_Identifier_Type);

      procedure Get_Auction_Info
        (Communication : out Auction_Communication_Ref;
         Bid : out Integer;
         Title : out String_Ref;
         Description : out String_Ref;
         Auction_Id : out Auction_Identifier_Type);

      entry Deposit (Amount : Natural);
      procedure Get_Amount (Amount : out Natural);

      entry Join_Auction (Communication : Auction_Communication_Ref;
                          Auction : Transactional_Auction_Ref);
      procedure Get_Join_Info
        (Communication : out Auction_Communication_Ref;
         Auction : out Transactional_Auction_Ref);

     private

      Current_Operation : User_Operation_Type := Nothing;
      Current_Initial_Bid : Integer;
      Current_Title : String_Ref;
      Current_Description : String_Ref;
      Current_Amount : Natural;
      Current_Communication : Auction_Communication_Ref;
      Current_Balance : Integer := -1;
      Current_Auction : Transactional_Auction_Ref;
      Current_Name : String_Ref;
      Current_Auction_Id : Auction_Identifier_Type;

   end User_Communication_Type;

   type User_Communication_Ref is access all User_Communication_Type;

end User_Communication;
