with My_Strings; use My_Strings;
with Debug_Auction; use Debug_Auction;
with Storage_Params.Non_Volatile.Non_Stable.File_Storage_Params;
use Storage_Params.Non_Volatile.Non_Stable.File_Storage_Params;
with Ada.Task_Identification; use Ada.Task_Identification;

with Object_Based_Transaction_Interface;
use Object_Based_Transaction_Interface;

with User_Objects; use User_Objects;
with User_Communication; use User_Communication;
with Auction_Objects; use Auction_Objects;
with Auction_Lists; use Auction_Lists;
with Transactional_Auctions; use Transactional_Auctions;

package body User_Interface is

   procedure Deposit (User   : in User_Identifier_Type;
                      Amount : in Natural) is
   begin
      User_Table (User).Communication.Deposit (Amount);
   end Deposit;

   function Balance (User : in User_Identifier_Type) return Natural is
      Result : Natural;
   begin
      User_Table (User).Communication.Get_Balance_Start;
      User_Table (User).Communication.Get_Balance_Result (Result);
      return Result;
   end Balance;

   function Get_Name (User : in User_Identifier_Type)
                     return String is
      Result : My_Strings.String_Ref;
   begin
      User_Table (User).Communication.Get_User_Name_Start;
      User_Table (User).Communication.Get_User_Name_Result (Result);
      return Result.all;
   end Get_Name;

   function Start_Auction (User : in User_Identifier_Type;
                           Title, Description : in String;
                           Initial_Bid : in Integer)
      return Auction_Identifier_Type is
      Result : Auction_Identifier_Type;
      Found : Boolean := False;
   begin

      for I in Auction_Table'Range loop
         if Auction_Table (I).Taken and then
           Auction_Table (I).Auctionname.S = Title then
            raise Auction_Exists;
         end if;
      end loop;

      for I in Auction_Table'Range loop
         if not Auction_Table (I).Taken then
           Auction_Table (I).Auctionname := To_String_Type (Title);
           Auction_Table (I).Taken := True;
           Auction_Table (I).User_Id := User;
           Result := I;
           Found := True;
           exit;
         end if;
      end loop;

      if not Found then
         raise Too_Much_Auctions;
      end if;

      User_Table (User).Communication.Start_Auction (Auction_Table (Result).
                                                     Communication'Access,
                                                     Title, Description,
                                                     Initial_Bid, Result);
      Debug_Auction.Put_Line ("Returning ID: " &
                              Auction_Identifier_Type'Image (Result));
      return Result;

   end Start_Auction;

   function Join_Auction (User : in User_Identifier_Type;
                          Auctionname : in String) return Auction_Identifier_Type is
      Result : Auction_Identifier_Type;
      Found : Boolean := False;
   begin

      for I in Auction_Table'Range loop
         if Auction_Table (I).Taken and then
           Auction_Table (I).Auctionname.S = Auctionname then
            Found := True;
            Result := I;
            exit;
         end if;
      end loop;

      if not Found then
         raise No_Such_Auction;
      end if;

      declare
         Bidder : Bidder_Node_Ref := Auction_Table (Result).Bidder_Nodes;
      begin
         while Bidder /= null loop
            if Bidder.User = User then
               return Result;
            end if;
            Bidder := Bidder.Next;
         end loop;
         Auction_Table (Result).Auction := The_Auctions.Get_Auction_Object (Auctionname);
         Bidder := new Bidder_Node;
         Bidder.User := User;
         Bidder.Next := Auction_Table (Result).Bidder_Nodes;
         Auction_Table (Result).Bidder_Nodes := Bidder;
         User_Table (User).Communication.Join_Auction
           (Bidder.Bidder_Communication'Access,
            Auction_Table (Result).Auction);
         Put_Line ("Added new bidder communication object");
      end;

      return Result;

   end Join_Auction;

end User_Interface;
