-------------------------------------------------------------------------------
--                             Accounts                                      --
--                                                                           --
--                        by Simon Schule,                                   --
--             Last modified 26.09.2000, by Xavier Caron                     --
--                      e-mail:  xavier.caron@epfl.ch                        --
--                                                                           --
--                                                                           --
--  The Account class contains the type for an user account of               --
--  the auction system.                                                      --
-------------------------------------------------------------------------------
with Ada.Streams;

package Accounts is

   type Account_Type is tagged limited private;
   type Account_Ref is access all Account_Type'Class;

   procedure Deposit (A : in out Account_Type;
                      Value : in Natural);

   procedure Withdraw (A : in out Account_Type;
                       Value : in Natural);

   function Balance (A : Account_Type) return Natural;

   Not_Enough_Funds : exception;

private

   type Account_Type is tagged limited record
      Balance : Natural := 0;
   end record;

   procedure My_Write (Stream : access Ada.Streams.Root_Stream_Type'Class;
                       Item   : in Account_Type);
   for Account_Type'Write use My_Write;

   procedure My_Read (Stream : access Ada.Streams.Root_Stream_Type'Class;
                      Item   : out Account_Type);
   for Account_Type'Read use My_Read;

end Accounts;
