-------------------------------------------------------------------------------
--                             Auction Lists (body)                          --
--                                                                           --
--                  by Xavier Caron, last modified 26.10.2000                --
--                                                                           --
--                      e-mail:  xavier.caron@epfl.ch                        --
--                                                                           --
--                                                                           --
--  The Auction Lists class contains the type for a list of all auctions     --
--  of the auction system.                                                   --
-------------------------------------------------------------------------------
with Ada.Unchecked_Deallocation;
with Remote_String_Arrays; use Remote_String_Arrays;
with Debug_Auction;
package body Auction_Lists is

   procedure Free is new Ada.Unchecked_Deallocation (Auction_Node_Type,
                                                     Auction_Node_Ref);

   protected body Auctions_List is

      procedure Insert_Auction (Auction : Transactional_Auction_Ref;
                                Title : String;
                                Auction_Id : Auction_Identifier_Type) is

         Tmp : Auction_Node_Ref := My_List;
         New_Auction : Auction_Node_Ref;
      begin
         New_Auction := new Auction_Node_Type;
         New_Auction.My_Auction := Auction;
         New_Auction.My_Name := new String'(Title);
         New_Auction.My_Auction_Id := Auction_Id;

         if Tmp = null then  --  If list empty
            My_List := New_Auction;
         else
            New_Auction.Next := My_List;
            My_List := New_Auction;
         end if;
         Count := Count + 1;

      end Insert_Auction;

      procedure Remove_Auction (Auction : Transactional_Auction_Ref)
      is
         Tmp : Auction_Node_Ref := My_List;
         Previous : Auction_Node_Ref := My_List;
      begin
         for X in 1 .. Count loop
            if Tmp.My_Auction = Auction then
               Free (Tmp.My_Name);
               if X = 1 then
                  My_List := Tmp.Next;
               else
                  Previous.Next := Tmp.Next;
               end if;
               Count := Count - 1;
               Free (Tmp);
               exit;
            end if;
            Previous := Tmp;
            Tmp := Tmp.Next;
         end loop;
      end Remove_Auction;

      function Get_All_Auctions return Auction_Array_Type is
         Tmp : Auction_Node_Ref := My_List;
         Result : Auction_Array_Type (1 .. Count);
      begin
         for X in 1 .. Count loop
            Result (X) := Tmp.My_Auction;
            Tmp := Tmp.Next;
         end loop;
         return Result;
      end Get_All_Auctions;

      function Get_All_Auctions_Titles return Remote_String_Array_Type is
         Tmp : Auction_Node_Ref := My_List;
         Result : Remote_String_Array_Type (Count);
      begin
         for X in 1 .. Count loop
            declare
               Title : String_Type (Tmp.My_Name'Length);
            begin
               Title.S := Tmp.My_Name.all;
               Result.Elements (X) := Title;
               Result.Ids (X) := Tmp.My_Auction_Id;
               Tmp := Tmp.Next;
            end;
         end loop;
         return Result;
      end Get_All_Auctions_Titles;

      function Get_Auction_Object (Name : String) return Transactional_Auction_Ref is
         Tmp : Auction_Node_Ref := My_List;
      begin
         Debug_Auction.Put_Line ("Looking for auction:" & Name);
         for X in 1 .. Count loop
            Debug_Auction.Put_Line ("Loop:" & My_List.My_Name.all);
            if Name = Tmp.My_Name.all then
               return Tmp.My_Auction;
            end if;
            Tmp := Tmp.Next;
         end loop;
         raise No_Such_Auction;
      end Get_Auction_Object;

   end Auctions_List;

   --  Load and Save

   --   procedure Load_List (Auction_List : out Auction_List_Type;
   --                         Stream : Abstract_Stream_Ref) is
   --    begin
   --       null;
   --    end Load_List;

   --    procedure Save_List (Auction_List : in Auction_List_Type;
   --                         Stream : Abstract_Stream_Ref) is
   --    begin
   --       null;
   --    end Save_List;


   --  procedure Finalize (Auction_List : in out Auction_List_Type) is
   --       Tmp : Auction_Node_Ref := Auction_List.My_List;
   --       Tmp2 : Auction_Node_Ref := Auction_List.My_List;
   --    begin
   --       while Tmp /= null loop
   --          Tmp2 := Tmp;
   --          Tmp := Tmp.Next;
   --          Free (Tmp2);
   --       end loop;
   --    end Finalize;

end Auction_Lists;
