-------------------------------------------------------------------------------
--                             Auction Lists                                 --
--                                                                           --
--                  by Xavier Caron, last modified 26.10.2000                --
--                                                                           --
--                      e-mail:  xavier.caron@epfl.ch                        --
--                                                                           --
--                                                                           --
--  The Auction Lists class contains the type for a list of all auctions     --
--  of the auction system. This type is not transactional!!!                 --
-------------------------------------------------------------------------------

with Transactional_Auctions; use Transactional_Auctions;
with My_Strings; use My_Strings;
with Remote_String_Arrays; use Remote_String_Arrays;
with Auction_Interface; use Auction_Interface;
--  with Streams; use Streams;

package Auction_Lists is

   type Auction_Array_Type is array (Integer range <>)
     of Transactional_Auction_Ref;

 --    type Auction_Id_Array_Type is array (Integer range <>)
--       of Auction_Identifier_Type;

   type Auction_Node_Type is private;
   type Auction_Node_Ref is access Auction_Node_Type;

   protected type Auctions_List is

      procedure Insert_Auction (Auction : Transactional_Auction_Ref;
                                Title : String;
                                Auction_Id : Auction_Identifier_Type);
      procedure Remove_Auction (Auction : Transactional_Auction_Ref);

      function Get_All_Auctions return Auction_Array_Type;
      function Get_All_Auctions_Titles return Remote_String_Array_Type;

      function Get_Auction_Object (Name : String) return Transactional_Auction_Ref;

   private
      My_List : Auction_Node_Ref := null;
      Count : Natural := 0;
   end Auctions_List;

   The_Auctions : Auctions_List;
   Finished_Auctions : Auctions_List;

   No_Such_Auction : exception;

private
   type Auction_Node_Type is record
      My_Auction : Transactional_Auction_Ref;
      My_Name : String_Ref;
      My_Auction_Id : Auction_Identifier_Type;
      Next : Auction_Node_Ref := null;
   end record;

   --  Load and Save

   --    procedure Load_List (Auction_List : out Auction_List_Type;
   --                         Stream : Abstract_Stream_Ref);

   --    procedure Save_List (Auction_List : in Auction_List_Type;
   --                         Stream : Abstract_Stream_Ref);
   --  Exceptions

end Auction_Lists;
