-------------------------------------------------------------------------------
--                              Auctions (body part)                         --
--                                                                           --
--                  by Xavier Caron, last modified 26.09.2000                --
--                                                                           --
--                      e-mail:  xavier.caron@epfl.ch                        --
--                                                                           --
--                                                                           --
--  The Auction class contains the type for an auction of the auction system --
-------------------------------------------------------------------------------
with Debug_Auction;
package body Auctions is

   function Create_Auction (Name : String;
                            Description : String;
                            Initial_Bid : Natural;
                            Seller_Task_Id : Task_Id)
                           return Auction_Ref is

   begin
      declare
         New_Auction : Auction_Ref :=
          new Auction_Type (Name'Length, Description'Length);
      begin
         New_Auction.Name := Name;
         New_Auction.Description := Description;
         New_Auction.Initial_Bid := Initial_Bid;
         New_Auction.Seller_Task_Id := Seller_Task_Id;
         return New_Auction;
      end;
   end Create_Auction;

   procedure Bid (Auction : in out Auction_Type; New_Bid : Natural) is
   begin
      if Auction.Current_Bid = 0 and then
        New_Bid >= Auction.Initial_Bid then
         Auction.Current_Bid := New_Bid;
         Auction.Current_Task_Id := Current_Task;
      elsif Auction.Current_Bid < New_Bid then
         Auction.Current_Bid := New_Bid;
         Auction.Current_Task_Id := Current_Task;
      else
         raise Bid_Refused;
      end if;
   end Bid;

   function Is_Accepted (Auction : Auction_Type) return Boolean is
   begin
      return Current_Task = Auction.Current_Task_Id;
   end Is_Accepted;

   function Status (Auction : Auction_Type) return Auction_Status is
   begin
      if Auction.Status = Accepted then
         if Current_Task = Auction.Current_Task_Id then
            return Winner;
         else
            return Loser;
         end if;
      else
         if Current_Task = Auction.Current_Task_Id then
            return Current_Winner;
         else
            return Auction.Status;
         end if;
      end if;
   end Status;

   procedure Set_Status (Auction : in out Auction_Type;
                         Status : Auction_Status) is
   begin
      Auction.Status := Status;
   end Set_Status;

   function Get_Current_Bid (Auction : Auction_Type) return Natural is
   begin
      return Auction.Current_Bid;
   end Get_Current_Bid;

   function Get_Initial_Bid (Auction : Auction_Type) return Natural is
   begin
      return Auction.Initial_Bid;
   end Get_Initial_Bid;

   function Get_Name (Auction : Auction_Type) return String is
   begin
      return Auction.Name;
   end Get_Name;

   function Get_Description (Auction : Auction_Type) return String is
   begin
      return Auction.Description;
   end Get_Description;

   function Get_Seller_Task_Id (Auction : Auction_Type) return Task_Id is
   begin
      return Auction.Seller_Task_Id;
   end Get_Seller_Task_Id;

   procedure Load_State (Auction : in out Auction_Type;
                         Stream : in Abstract_Stream_Ref) is
   begin
      Auction := Auction_Type'Input (Stream);
   end Load_State;

end Auctions;

