-------------------------------------------------------------------------------
--                             Auctions                                      --
--                                                                           --
--                  by Xavier Caron, last modified 26.09.2000                --
--                                                                           --
--                      e-mail:  xavier.caron@epfl.ch                        --
--                                                                           --
--                                                                           --
--  The Auction class contains the type an auction of the auction system.    --
-------------------------------------------------------------------------------
with Streams; use Streams;
with Ada.Task_Identification; use Ada.Task_Identification;
with Ada.Task_Identification; use Ada.Task_Identification;

package Auctions is

   type Auction_Status is (Active, Current_Winner, Winner, Loser, Accepted);

   type Auction_Type (<>) is private;
   type Auction_Ref is access all Auction_Type;

   function Create_Auction (Name : String;
                            Description : String;
                            Initial_Bid : Natural;
                            Seller_Task_Id : Task_Id)
                           return Auction_Ref;

   procedure Bid (Auction : in out Auction_Type; New_Bid : Natural);

   function Is_Accepted (Auction : Auction_Type) return Boolean;

   function Status (Auction : Auction_Type) return Auction_Status;
   procedure Set_Status (Auction : in out Auction_Type;
                         Status : Auction_Status);

   function Get_Current_Bid (Auction : Auction_Type) return Natural;
   function Get_Initial_Bid (Auction : Auction_Type) return Natural;


   function Get_Name (Auction : Auction_Type) return String;
   function Get_Description (Auction : Auction_Type) return String;

   function Get_Seller_Task_Id (Auction : Auction_Type) return Task_Id;

   procedure Load_State (Auction : in out Auction_Type;
                         Stream : in Abstract_Stream_Ref);

   Bid_Refused : exception;

private

   type Auction_Type (N, D : Natural) is record
      Name : String (1 .. N);
      Description : String (1 .. D);
      Initial_Bid : Natural;
      Current_Bid : Natural := 0;
      Seller_Task_Id : Task_Id := Null_Task_Id;
      Current_Task_Id : Task_Id := Null_Task_Id;
      Status : Auction_Status := Active;
   end record;

end Auctions;

