-------------------------------------------------------------------------------
--                      Transactional Accounts                               --
--                                                                           --
--                        by Simon Schule,                                   --
--             Last modified 26.09.2000, by Xavier Caron                     --
--                      e-mail:  xavier.caron@epfl.ch                        --
--                                                                           --
--                                                                           --
--  The Transactional Account class contains the type for an user account of --
--  the auction system.                                                      --
-------------------------------------------------------------------------------

with Accounts; use Accounts;
with Atomic_Calls; use Atomic_Calls;
with Memory_Objects.Concrete; use Memory_Objects.Concrete;
with Storage_Params.Non_Volatile; use Storage_Params.Non_Volatile;

package Transactional_Accounts is

   type Transactional_Account_Type is tagged limited private;
   type Transactional_Account_Ref is
     access all Transactional_Account_Type'Class;

   procedure Deposit (A : in out Transactional_Account_Type;
                      Value : in Natural);

   procedure Withdraw (A : in out Transactional_Account_Type;
                       Value : in Natural);

   function Balance (A : Transactional_Account_Type) return Natural;

   function Create_Account (Params : in Non_Volatile_Params_Type'Class)
                           return Transactional_Account_Ref;

   function Restore_Account (Params : in Non_Volatile_Params_Type'Class)
                            return Transactional_Account_Ref;

   Not_Enough_Funds : exception renames Accounts.Not_Enough_Funds;

private

   type Transactional_Account_Type is tagged limited record
      My_Atomic_Call : Atomic_Call_Ref;
      My_Memory_Object : Concrete_Memory_Object_Ref;
   end record;

end Transactional_Accounts;

