-------------------------------------------------------------------------------
--                        Transactional Auctions                             --
--                                                                           --
--                  by Xavier Caron, last modified 18.10.2000                --
--                                                                           --
--                      e-mail:  xavier.caron@epfl.ch                        --
--                                                                           --
--                                                                           --
--  The Transactional Auction class contains the type auction of the         --
--  auction system.                                                          --
-------------------------------------------------------------------------------

with Atomic_Calls; use Atomic_Calls;
with Memory_Objects.Concrete; use Memory_Objects.Concrete;
with Storage_Params.Non_Volatile; use Storage_Params.Non_Volatile;
with Ada.Task_Identification; use Ada.Task_Identification;
with Auctions; use Auctions;

package Transactional_Auctions is

   type Transactional_Auction_Type (<>) is tagged limited private;
   type Transactional_Auction_Ref is access all Transactional_Auction_Type;

   function Create_Auction (Params : in Non_Volatile_Params_Type'Class;
                            Name : String;
                            Description : String;
                            Initial_Bid : Natural;
                            Seller_Task_Id : Task_Id)
                           return Transactional_Auction_Ref;

   function Restore_Auction (Params : in Non_Volatile_Params_Type'Class)
                            return Transactional_Auction_Ref;

   procedure Place_Bid (Auction : Transactional_Auction_Type; New_Bid : Natural);

   function Is_Accepted (Auction : Transactional_Auction_Type)
                        return Boolean;

   function Status (Auction : Transactional_Auction_Type)
                   return Auction_Status;

   procedure Set_Status (Auction : Transactional_Auction_Type;
                         Status : Auction_Status);

   function Get_Current_Bid (Auction : Transactional_Auction_Type)
                            return Natural;

   function Get_Initial_Bid (Auction : Transactional_Auction_Type)
                            return Natural;

   function Get_Name (Auction : Transactional_Auction_Type)
                     return String;

   function Get_Description (Auction : Transactional_Auction_Type)
                            return String;

   function Get_Seller_Task_Id (Auction : Transactional_Auction_Type)
                               return Task_Id;

private

   type Transactional_Auction_Type is tagged limited record
      My_Atomic_Call : Atomic_Call_Ref;
      My_Memory_Object : Concrete_Memory_Object_Ref;
   end record;

end Transactional_Auctions;

