-------------------------------------------------------------------------------
--                       Transactional Users Directories                     --
--                                                                           --
--                  by Xavier Caron, last modified 06.10.2000                --
--                                                                           --
--                      e-mail:  xavier.caron@epfl.ch                        --
--                                                                           --
--                                                                           --
--  The Transactional Users Directories class contains the type for the      --
--  user dir of the auction system.                                          --
-------------------------------------------------------------------------------
with Atomic_Calls; use Atomic_Calls;
with Memory_Objects.Concrete; use Memory_Objects.Concrete;
with Storage_Params.Non_Volatile; use Storage_Params.Non_Volatile;
with Transactional_Users;
use Transactional_Users;
with User_Directories;
use User_Directories;

package Transactional_User_Directories is

   type Transactional_User_Directory_Type (<>) is tagged limited private;
   type Transactional_User_Directory_Ref is access all
     Transactional_User_Directory_Type'Class;

   procedure Register_User
     (User_Dir : in out Transactional_User_Directory_Type;
      Name : String;
      Password : String;
      Params : Non_Volatile_Params_Type'Class);

   function Get_User (User_Dir : Transactional_User_Directory_Type;
                      Name : String; Password : String)
                     return Transactional_User_Ref;

   function Get_All_Users (User_Dir : Transactional_User_Directory_Type)
                          return String_Array_Type;

   function Create_User_Directory (Params : in Non_Volatile_Params_Type'Class)
                                  return Transactional_User_Directory_Ref;

   function Restore_User_Directory (Params : in Non_Volatile_Params_Type'Class)
                                   return Transactional_User_Directory_Ref;

private

   type Transactional_User_Directory_Type is tagged limited record
      My_Atomic_Call : Atomic_Call_Ref;
      My_Memory_Object : Concrete_Memory_Object_Ref;
   end record;

end Transactional_User_Directories;

