-------------------------------------------------------------------------------
--                            Transactional Users                            --
--                                                                           --
--                  by Xavier Caron, last modified 26.09.2000                --
--                                                                           --
--                      e-mail:  xavier.caron@epfl.ch                        --
--                                                                           --
--                                                                           --
--  The Transactional Users class contains the type for the user of the      --
--  auction system.                                                          --
-------------------------------------------------------------------------------
with Atomic_Calls; use Atomic_Calls;
with Memory_Objects.Concrete; use Memory_Objects.Concrete;
with Storage_Params.Non_Volatile; use Storage_Params.Non_Volatile;
with Transactional_Accounts; use Transactional_Accounts;

package Transactional_Users is

   type Transactional_User_Type (<>) is tagged limited private;
   type Transactional_User_Ref is access all Transactional_User_Type'Class;

   function Get_Name (User : Transactional_User_Type) return String;

   function Get_Account (User : Transactional_User_Type)
                        return Transactional_Account_Ref;

   function Create_User (Params : in Non_Volatile_Params_Type'Class;
                         Name : String;
                         Account_Params : in Non_Volatile_Params_Type'Class)
                        return Transactional_User_Ref;

   function Restore_User (Params : in Non_Volatile_Params_Type'Class)
     return Transactional_User_Ref;

private

   type Transactional_User_Type is tagged limited record
      My_Atomic_Call : Atomic_Call_Ref;
      My_Memory_Object : Concrete_Memory_Object_Ref;
   end record;

end Transactional_Users;

