-------------------------------------------------------------------------------
--                             User Directories                              --
--                                                                           --
--                  by Xavier Caron, last modified 29.09.2000                --
--                                                                           --
--                      e-mail:  xavier.caron@epfl.ch                        --
--                                                                           --
--                                                                           --
--  The Users Directory class contains the type for a list of all users      --
--  of the auction system.                                                   --
-------------------------------------------------------------------------------
with Storage_Params.Non_Volatile; use Storage_Params.Non_Volatile;

with Transactional_Users; use Transactional_Users;

with Ada.Finalization;
with Streams; use Streams;

package User_Directories is

   type User_Directory_Type is limited private;

   type User_Directory_Ref is access all User_Directory_Type;

   procedure Register_User (User_Dir : in out User_Directory_Type;
                            Name : String;
                            Password : String;
                            Params : Non_Volatile_Params_Type'Class);

   procedure Remove_User (User_Dir : in out User_Directory_Type;
                          Name : String);

   function Get_User (User_Dir : User_Directory_Type;
                      Name : String; Password : String)
     return Transactional_User_Ref;

   --  To retrieve all users

   --  String type

   type String_Ref is access String;

   type String_Type is new Ada.Finalization.Controlled with record
      S : String_Ref;
   end record;

   procedure Finalize (S : in out String_Type);

   type String_Array_Type is array (Integer range <>) of String_Ref;

   function Get_All_Users (User_Dir : User_Directory_Type)
                          return String_Array_Type;

   --  Load and Save

   procedure Load_Directory (User_Dir : out User_Directory_Type;
                         Stream : Abstract_Stream_Ref);

   procedure Save_Directory (User_Dir : in User_Directory_Type;
                             Stream : Abstract_Stream_Ref);
   --  Exceptions

   Login_Incorrect, User_Already_Existant, User_Not_Existant : exception;

private

   --  Linked list of users.

   type User_Node_Type;

   type User_Node_Ref is access User_Node_Type;

   type User_Node_Type is record
      My_Name : String_Type;
      My_Password : String_Type;
      My_Params : Non_Volatile_Params_Ref;
      Next : User_Node_Ref := null;
   end record;

   type User_Directory_Type is new Ada.Finalization.Limited_Controlled with
      record
         My_List : User_Node_Ref := null;
         Count : Natural := 0;
      end record;

   procedure Finalize (User_Dir : in out User_Directory_Type);

end User_Directories;


