-------------------------------------------------------------------------------
--                              Users (body part)                            --
--                                                                           --
--                  by Xavier Caron, last modified 26.09.2000                --
--                                                                           --
--                      e-mail:  xavier.caron@epfl.ch                        --
--                                                                           --
--                                                                           --
--  The Users class contains the type for the user of the auction system.    --
-------------------------------------------------------------------------------

with Ada.Unchecked_Deallocation;

package body Users is

   function Create_User (Name : String;
                         Params : Non_Volatile_Params_Type'Class)
                        return User_Ref is
      New_User : User_Ref := new User_Type (Name'Length);
   begin
      New_User.Name := Name;
      New_User.Params.Account_Params :=
        new Non_Volatile_Params_Type'Class' (Params);
      return New_User;
   end Create_User;

   function Get_Name (User : User_Type) return String is
   begin
      return User.Name;
   end Get_Name;

   function Get_Account (User : User_Type)
                        return Transactional_Account_Ref is
   begin
      return Restore_Account (User.Params.Account_Params.all);
   end Get_Account;

   procedure Load_State (User : in out User_Type;
                         Stream : in Abstract_Stream_Ref) is
   begin
      User := User_Type'Input (Stream);
   end Load_State;

   procedure My_Write (Stream : access Ada.Streams.Root_Stream_Type'Class;
                       Item   : in Compiler_Bug_Avoider_Type) is
   begin
      Non_Volatile_Params_Type'Class'Output (Stream, Item.Account_Params.all);
   end My_Write;

   procedure My_Read (Stream : access Ada.Streams.Root_Stream_Type'Class;
                      Item   : out Compiler_Bug_Avoider_Type) is

      procedure Free is new Ada.Unchecked_Deallocation
        (Non_Volatile_Params_Type'Class, Non_Volatile_Params_Ref);

   begin
      Free (Item.Account_Params);
      Item.Account_Params := new Non_Volatile_Params_Type'Class'
        (Non_Volatile_Params_Type'Class'Input (Stream));
   end My_Read;

end Users;

