-------------------------------------------------------------------------------
--                             Users                                         --
--                                                                           --
--                  by Xavier Caron, last modified 26.09.2000                --
--                                                                           --
--                      e-mail:  xavier.caron@epfl.ch                        --
--                                                                           --
--                                                                           --
--  The Users class contains the type for the user of the auction system.    --
-------------------------------------------------------------------------------

with Ada.Streams;
with Streams; use Streams;
with Storage_Params.Non_Volatile; use Storage_Params.Non_Volatile;
with Transactional_Accounts; use Transactional_Accounts;

package Users is

   type User_Type (<>) is private;
   type User_Ref is access all User_Type;

   function Create_User (Name : String;
                         Params : Non_Volatile_Params_Type'Class)
                        return User_Ref;

   function Get_Name (User : User_Type) return String;

   function Get_Account (User : User_Type)
                        return Transactional_Account_Ref;

   procedure Load_State (User : in out User_Type;
                         Stream : in Abstract_Stream_Ref);

private

   type Compiler_Bug_Avoider_Type is record
      Account_Params : Non_Volatile_Params_Ref;
   end record;

   procedure My_Write (Stream : access Ada.Streams.Root_Stream_Type'Class;
                       Item   : in Compiler_Bug_Avoider_Type);
   for Compiler_Bug_Avoider_Type'Write use My_Write;

   procedure My_Read (Stream : access Ada.Streams.Root_Stream_Type'Class;
                      Item   : out Compiler_Bug_Avoider_Type);
   for Compiler_Bug_Avoider_Type'Read use My_Read;

   type User_Type (N : Natural) is record
      Name : String (1 .. N);
      Params : Compiler_Bug_Avoider_Type;
   end record;

end Users;
