with Object_Based_Transaction_Interface;
with Procedural_Transaction_Interface;
with Auctions; use Auctions;
with Transactional_Auctions; use Transactional_Auctions;
with Debug; use Debug;
with Auction_Lists; use Auction_Lists;
with Ada.Task_Identification; use Ada.Task_Identification;
with Storage_Params.Non_Volatile.Non_Stable.File_Storage_Params;
use Storage_Params.Non_Volatile.Non_Stable.File_Storage_Params;
with Auction_Communication; use Auction_Communication;
with Auction_Lists; use Auction_Lists;

with Ada.Exceptions; use Ada.Exceptions;
with My_Strings; use My_Strings;

package body Bidder_Tasks is

   task body Bidder_Task_Type is
      Op : Auction_Operation_Type;
      Title : String_Ref;
      Description : String_Ref;
      Username : String_Ref;

   begin

      Put_Line ("Bidder task: started...", Auction);
      declare
         T : Object_Based_Transaction_Interface.Transaction;
      begin
         Title := new String' (Get_Name (My_Auction.all));
         Description := new String' (Get_Description (My_Auction.all));
         Username := new String' (Get_Name (My_User.all));
         Object_Based_Transaction_Interface.Commit_Transaction (T);
      end;

      declare
         T : Object_Based_Transaction_Interface.Transaction
           (Object_Based_Transaction_Interface.String_Ref (Title), null, 0);
         Current_Status : Auction_Status := Auctions.Active;
         Subtransaction : Boolean := False;
      begin
         Put_Line ("Bidder task " & Username.all & "-" & Title.all &
                   ": joined transaction " & Title.all, Auction);

         while Current_Status /= Auctions.Loser and
           Current_Status /= Auctions.Winner loop
            Put_Line ("Bidder task " & Username.all & "-" & Title.all &
                      ": Waiting for command");

            Communication.Called (Op);
            Put_Line ("Bidder task " & Username.all & "-" & Title.all & ": Handling "
                      & Auction_Operation_Type'Image (Op));
            case Op is

               when Get_Initial_Bid =>
                  declare
                     Bid : Natural := Get_Initial_Bid (My_Auction.all);
                  begin
                     Communication.Set_Initial_Bid (Bid);
                  end;

               when Get_Current_Bid =>
                  declare
                     Bid : Natural := Get_Current_Bid (My_Auction.all);
                  begin
                     Communication.Set_Current_Bid (Bid);
                  end;

               when Get_Auction_Name =>
                  Communication.Set_Auction_Name (Title);

               when Get_Auction_Description =>
                  Communication.Set_Auction_Description (Description);

               when Get_Status =>
                  declare
                     Bid : Natural := Get_Current_Bid (My_Auction.all);
                  begin
                     Current_Status :=
                       Transactional_Auctions.Status (My_Auction.all);
                     Communication.Set_Status (Bid, Current_Status);
                     if Subtransaction and Current_Status = Active then
                        Put_Line ("Bidder task " & Username.all & "-" & Title.all
                                  & ": someone has placed a higher bid", Auction);
                        Procedural_Transaction_Interface.Abort_Transaction;
                        Put_Line ("Bidder task " & Username.all & "-" & Title.all
                                  & ": subtransaction aborted", Auction);
                        Subtransaction := False;
                     end if;
                  end;

               when Place_Bid =>
                  declare
                     Bid : Natural;
                  begin
                     Communication.Get_Bid (Bid);
                     Put_Line ("Bidder task " & Username.all & "-" & Title.all & ": got bid" &
                               Natural'Image (Bid));
                     Put_Line ("Bidder task " & Username.all & "-" & Title.all
                               & ": starting subtransaction...", Auction);
                     Procedural_Transaction_Interface.Begin_Transaction;
                     Subtransaction := True;
                     Put_Line ("Bidder task " & Username.all & "-" & Title.all
                               & ": withdrawing" & Natural'Image (Bid) & " from user account", Auction);
                     Withdraw (My_Account.all, Bid);
                     Place_Bid (My_Auction.all, Bid);
                     Put_Line ("Bidder task " & Username.all & "-" & Title.all & ": placed bid", Auction);
                     Communication.Place_Bid_Complete (True);
                  exception
                     when Transactional_Accounts.Not_Enough_Funds =>
                        Put_Line ("Bidder task " & Username.all & "-" & Title.all
                                  & ": not enought funds", Auction);
                        Procedural_Transaction_Interface.Abort_Transaction;
                        Put_Line ("Bidder task " & Username.all & "-" & Title.all
                                  & ": subtransaction aborted", Auction);
                        Subtransaction := False;
                        Communication.Place_Bid_Complete (False);
                  end;

               when others =>
                  null;

            end case;

         end loop;

         if Current_Status = Winner then
            Put_Line ("Bidder task " & Username.all & "-" &
                      Title.all & ": committing subtransaction", Auction);
            Procedural_Transaction_Interface.Commit_Transaction;
         elsif Subtransaction then
            Put_Line ("Bidder task " & Username.all & "-" &
                      Title.all & ": aborting subtransaction", Auction);
            Procedural_Transaction_Interface.Abort_Transaction;
         end if;
         Subtransaction := False;

         Put_Line ("Bidder task " & Username.all & "-" & Title.all
                   & ": committing transaction "
                   & Title.all, Auction);
         Object_Based_Transaction_Interface.Commit_Transaction (T);
      end;

      Put_Line ("Bidder task " & Username.all & "-" & Title.all & ": terminating...", Auction);

      declare
         Tmp_Title : String_Ref := Title;
         Tmp_Description : String_Ref := Description;
         Tmp_User : String_Ref := Username;
      begin
         Free (Tmp_Title);
         Free (Description);
         Free (Tmp_User);
      end;

   exception
      when E : others =>
         Put_Line ("Bidder task " & Username.all & "-" & Title.all
                   & ": raised " & Exception_Name (E));
   end Bidder_Task_Type;

end Bidder_Tasks;

