with Object_Based_Transaction_Interface;

with Auctions; use Auctions;
with Transactional_Auctions; use Transactional_Auctions;
with Debug; use Debug;
with Auction_Lists; use Auction_Lists;
with Ada.Task_Identification; use Ada.Task_Identification;
with Storage_Params.Non_Volatile.Non_Stable.File_Storage_Params;
use Storage_Params.Non_Volatile.Non_Stable.File_Storage_Params;
with Auction_Communication; use Auction_Communication;
with Auction_Lists; use Auction_Lists;

with Ada.Exceptions; use Ada.Exceptions;

package body Seller_Tasks is

   task body Seller_Task_Type is
      Op : Auction_Operation_Type;
   begin
      Put_Line ("Seller task " & Title.all & ": started...", Auction);

      declare
         My_Auction : Transactional_Auction_Ref;
         T : Object_Based_Transaction_Interface.Transaction
           (Object_Based_Transaction_Interface.String_Ref (Title), null, 0);
      begin
         Put_Line ("Seller task " & Title.all & ": started transaction "
                   & Title.all, Auction);

         My_Auction := Create_Auction
           (String_To_Storage_Params (Title.all & ".auc"),
            Title.all, Description.all,
            Initial_Bid, Current_Task);
         The_Auctions.Insert_Auction (My_Auction, Title.all, My_Auction_Id);

         Put_Line ("Seller task " & Title.all & ": created auction object"
                   & " and inserted into list", Auction);

         loop
            Communication.Called (Op);
            Put_Line ("Seller task " & Title.all & ": Handling "
                      & Auction_Operation_Type'Image (Op));
            case Op is

               when Get_Initial_Bid =>
                  declare
                     Bid : Natural := Get_Initial_Bid (My_Auction.all);
                  begin
                     Communication.Set_Initial_Bid (Bid);
                  end;

               when Get_Current_Bid =>
                  declare
                     Bid : Natural := Get_Current_Bid (My_Auction.all);
                  begin
                     Communication.Set_Current_Bid (Bid);
                  end;

               when Get_Auction_Name =>
                  Communication.Set_Auction_Name (Title);

               when Get_Auction_Description =>
                  Communication.Set_Auction_Description (Description);

               when Accept_Bid =>
                  Set_Status (My_Auction.all, Accepted);
                  exit;

               when others =>
                  null;
            end case;
         end loop;

         Put_Line ("Seller task " & Title.all & ": removing auction from list");
         The_Auctions.Remove_Auction (My_Auction);

         Put_Line ("Seller task " & Title.all & ": deposit money on user account",
                   Auction);
         declare
            Bid : Natural := Get_Current_Bid (My_Auction.all);
         begin
            if Bid > 0 then
               Deposit (My_Account.all, Bid);
            end if;
         end;

         Put_Line ("Seller task " & Title.all & ": committing transaction " & Title.all);
         Object_Based_Transaction_Interface.Commit_Transaction (T);
      end;

      Put_Line ("Seller task " & Title.all & ": terminating...", Auction);

      declare
         Tmp_Title : String_Ref := Title;
         Tmp_Description : String_Ref := Description;
      begin
         Free (Tmp_Title);
         Free (Tmp_Description);
      end;

   exception
      when E : others =>
         Put_Line ("Seller task " & Title.all
                   & ": raised " & Exception_Name (E), User);
   end Seller_Task_Type;

end Seller_Tasks;

