with Object_Based_Transaction_Interface;

with Transactional_Auctions; use Transactional_Auctions;
with Debug_Auction; use Debug_Auction;
with Ada.Task_Identification; use Ada.Task_Identification;
with Storage_Params.Non_Volatile.Non_Stable.File_Storage_Params;
use Storage_Params.Non_Volatile.Non_Stable.File_Storage_Params;

with Seller_Tasks; use Seller_Tasks;
with Bidder_Tasks; use Bidder_Tasks;

with Auction_Communication; use Auction_Communication;
--  with Auction_Interface; use Auction_Interface;
with Remote_String_Arrays; use Remote_String_Arrays;

with Ada.Exceptions; use Ada.Exceptions;

package body User_Tasks is

   task body User_Task_Type is
      Op : User_Operation_Type;
      User_Name : String_Ref;

   begin

      Put_Line ("User task " & Username.all & ": started...");
      declare
         T : Object_Based_Transaction_Interface.Transaction;
      begin
         User_Name := new String'(Get_Name (My_User.all));
         Object_Based_Transaction_Interface.Commit_Transaction (T);
      end;

      loop
      Communication.Called (Op);
      Put_Line ("User task " & Username.all & ": handling operation "
                & User_Operation_Type'Image (Op));

      case Op is

         when Deposit =>
            declare
               Amount : Natural;
               T : Object_Based_Transaction_Interface.Transaction;
            begin
               Communication.Get_Amount (Amount);
               Transactional_Accounts.Deposit (My_Account.all, Amount);
               Put_Line ("User task " & Username.all & ": Finished deposit");
               Object_Based_Transaction_Interface.Commit_Transaction (T);
            end;

         when Balance =>
            declare
               T : Object_Based_Transaction_Interface.Transaction;
               Balance : Natural :=
                 Transactional_Accounts.Balance
                 (My_Account.all);
            begin
               Communication.Set_Balance (Balance);
               Put_Line ("User task " & Username.all & ": Finished balance");
               Object_Based_Transaction_Interface.Commit_Transaction (T);
            end;

         when Join_Auction =>
            declare
               Auction_Communication : Auction_Communication_Ref;
               My_Auction : Transactional_Auction_Ref;
               T : Bidder_Task_Ref;
            begin
               Communication.Get_Join_Info (Auction_Communication,
                                            My_Auction);
               T := new Bidder_Task_Type (Auction_Communication,
                                          My_Auction, My_User, My_Account);
            end;
            Put_Line ("User task " & Username.all & ": bidder task started");

         when Get_User_Name =>
            Communication.Set_User_Name (User_Name);

         when Start_Auction =>

            declare
               Auction_Communication : Auction_Communication_Ref;
               Initial_Bid : Natural;
               Title, Description : String_Ref;
               Auction_Id : Auction_Identifier_Type;
               T : Seller_Task_Ref;
            begin
               Communication.Get_Auction_Info (Auction_Communication,
                                               Initial_Bid, Title,
                                               Description, Auction_Id);
               T := new Seller_Task_Type (Auction_Communication, Title,
                                          Description,
                                          Initial_Bid, My_User, My_Account, Auction_Id);
            end;
            Put_Line ("User task " & Username.all & ": Seller task started");

         when others => null;

      end case;
      end loop;
      Free (User_Name);

   exception
      when E : others =>
         Debug_Auction.Put_Line ("User task " & Username.all
                                 & ": raised " & Exception_Name (E));
   end User_Task_Type;

end User_Tasks;

