-------------------------------------------------------------------------------
--                            Server Utilities (body)                        --
--                                                                           --
--                  by Xavier Caron, last modified 08.03.2001                --
--                                                                           --
--                      e-mail:  xavier.caron@epfl.ch                        --
--                                                                           --
--  This package provide utilities for the server of the DASK                --
-------------------------------------------------------------------------------
with Object_Based_Transaction_Interface;
use Object_Based_Transaction_Interface;

with User_Directories; use User_Directories;
with Transactional_User_Directories; use Transactional_User_Directories;
with Storage_Params.Non_Volatile.Non_Stable.File_Storage_Params;
use Storage_Params.Non_Volatile.Non_Stable.File_Storage_Params;

with Debug_Auction;

package body Server_Utilities is

   procedure Open_Directory
     (Directory : in out Transactional_User_Directory_Ref);

   procedure Open_Directory
     (Directory : in out Transactional_User_Directory_Ref) is
      File_Params : File_Storage_Params_Type :=
        String_To_Storage_Params ("directory.dir");
   begin
      Directory := Restore_User_Directory (File_Params);
   exception
      when Object_Non_Existant =>
         Directory := Create_User_Directory (File_Params);
   end Open_Directory;

   function Login_Correct (Name : String; Password : String)
                          return Boolean is

      Temp_Dir : Transactional_User_Directory_Ref;
      Tmp_User : Transactional_User_Ref;
      T : Transaction;
   begin
      Open_Directory (Temp_Dir);
      Tmp_User := Get_User (Temp_Dir.all,
                            Name, Password);
      Commit_Transaction (T);
      return True;

   exception
      when Login_Incorrect =>
         Commit_Transaction (T);
         return False;
   end Login_Correct;

   procedure Create_New_User (Name : String; Password : String) is

      Temp_Dir : Transactional_User_Directory_Ref;
      Temp_User : Transactional_User_Ref;
      Result : Transactional_Account_Ref;
      T : Transaction;
   begin
      Open_Directory (Temp_Dir);
      Temp_User :=
        Create_User (String_To_Storage_Params (Name & ".usr"), Name,
                     String_To_Storage_Params (Name & ".acc"));

      Result := Create_Account
        (String_To_Storage_Params (Name & ".acc"));

      Register_User
        (Temp_Dir.all, Name, Password,
         String_To_Storage_Params (Name & ".usr"));

      Commit_Transaction (T);
   exception
      when Object_Exists =>
         Commit_Transaction (T);
         raise;
      when others =>
            null;
   end Create_New_User;

   function Get_The_User (Name : String;
                          Password : String)
                         return Transactional_User_Ref is
      Temp_Dir : Transactional_User_Directory_Ref;
      T : Transaction;
      Result : Transactional_User_Ref;
   begin
      Open_Directory (Temp_Dir);
      Result :=
        Transactional_User_Directories.Get_User (Temp_Dir.all, Name, Password);
      Commit_Transaction (T);
      return Result;
   end Get_The_User;

   function Get_The_Account (User : Transactional_User_Ref)
                            return Transactional_Account_Ref is
      T : Transaction;
      Result : Transactional_Account_Ref := new Transactional_Account_Type;
   begin
      Result := Restore_Account
        (String_To_Storage_Params (Get_Name (User.all) & ".acc"));

      Commit_Transaction (T);
      return Result;
   end Get_The_Account;

end Server_Utilities;
